@@ -19,7 +19,7 @@ from sortedcontainers import SortedList, SortedListWithKey
 
 
 __all__ = ['Gauge', 'Momentum']
-__version__ = '0.1.5'
+__version__ = '0.1.6'
 
 
 # indices
@@ -214,8 +214,8 @@ class Gauge(object):
         prev_time, prev_value = determination[x - 1]
         time_delta = next_time - prev_time
         value_delta = next_value - prev_value
-        ratio = float(at - prev_time) / time_delta
-        value = prev_value + ratio * value_delta
+        rate = float(at - prev_time) / time_delta
+        value = prev_value + rate * value_delta
         velocity = value_delta / time_delta
         return (value, velocity)
 
@@ -448,25 +448,25 @@ class Gauge(object):
             yield time, method, momentum
         yield (+inf, None, None)
 
-    def walk_segs(self, number_or_gauge):
-        """Yields :class:`Segment`s on the graph from `number_or_gauge`.  If
+    def walk_lines(self, number_or_gauge):
+        """Yields :class:`Line`s on the graph from `number_or_gauge`.  If
         `number_or_gauge` is a gauge, the graph is the determination of the
-        gauge.  Otherwise, just a horizontal line which has the number as the
+        gauge.  Otherwise, just a Horizon line which has the number as the
         Y-intercept.
         """
         if isinstance(number_or_gauge, Gauge):
             determination = number_or_gauge.determination
             first, last = determination[0], determination[-1]
-            yield Segment(first[VALUE], 0, self.base[TIME], first[TIME])
+            if self.base[TIME] < first[TIME]:
+                yield Horizon(self.base[TIME], first[TIME], first[VALUE])
             zipped_determination = zip(determination[:-1], determination[1:])
             for (time1, value1), (time2, value2) in zipped_determination:
-                velocity = (value2 - value1) / (time2 - time1)
-                yield Segment(value1, velocity, time1, time2)
-            yield Segment(last[VALUE], 0, last[TIME], +inf)
+                yield Segment(time1, time2, value1, value2)
+            yield Horizon(last[TIME], +inf, last[VALUE])
         else:
             # just a number.
             value = number_or_gauge
-            yield Segment(value, 0, self.base[TIME], +inf)
+            yield Horizon(self.base[TIME], +inf, value)
 
     def determine(self):
         """Determines the transformations from the time when the value set to
@@ -476,22 +476,17 @@ class Gauge(object):
         velocity, velocities = 0, []
         bound, overlapped = None, False
         # boundaries.
-        ceil = Boundary(self.walk_segs(self.max), operator.lt)
-        floor = Boundary(self.walk_segs(self.min), operator.gt)
+        ceil = Boundary(self.walk_lines(self.max), operator.lt)
+        floor = Boundary(self.walk_lines(self.min), operator.gt)
         boundaries = [ceil, floor]
-        def clamp_by_boundaries(value, at):
-            if bound is None or overlapped:
-                value = min(value, ceil.seg.guess(at))
-                value = max(value, floor.seg.guess(at))
-            return value
         for boundary in boundaries:
             # skip past boundaries.
-            while boundary.seg.until <= since:
+            while boundary.line.until <= since:
                 boundary.walk()
             # check overflowing.
             if bound is not None:
                 continue
-            boundary_value = boundary.seg.guess(since)
+            boundary_value = boundary.line.guess(since)
             if boundary.cmp(boundary_value, value):
                 bound, overlapped = boundary, False
         for time, method, momentum in self.walk_events():
@@ -506,39 +501,38 @@ class Gauge(object):
                     walked_boundaries = boundaries
                 else:
                     # stop the loop if all boundaries have been proceeded.
-                    if all(b.seg.until >= until for b in boundaries):
+                    if all(b.line.until >= until for b in boundaries):
                         break
                     # choose the next boundary.
-                    boundary = min(boundaries, key=lambda b: b.seg.until)
+                    boundary = min(boundaries, key=lambda b: b.line.until)
                     boundary.walk()
                     walked_boundaries = [boundary]
                 # calculate velocity.
                 if bound is None:
                     velocity = sum(velocities)
                 elif overlapped:
-                    velocity = bound.best(sum(velocities), bound.seg.velocity)
+                    velocity = bound.best(sum(velocities), bound.line.velocity)
                 else:
                     velocity = sum(v for v in velocities if bound.cmp(v, 0))
                 # is still bound?
-                if overlapped and bound.cmp(velocity, bound.seg.velocity):
+                if overlapped and bound.cmp(velocity, bound.line.velocity):
                     bound, overlapped = None, False
                     again = True
                     continue
-                # current segment.
-                seg = Segment(value, velocity, since, until)
+                # current ray.
+                line = Ray(since, until, value, velocity)
                 if overlapped:
-                    bound_until = min(bound.seg.until, until)
+                    bound_until = min(bound.line.until, until)
                     if bound_until == +inf:
                         break
                     # released from the boundary.
-                    since, value = (bound_until, seg.get(bound_until))
-                    value = clamp_by_boundaries(value, at=since)
+                    since, value = (bound_until, bound.line.get(bound_until))
                     yield (since, value)
                     continue
                 for boundary in walked_boundaries:
                     # find the intersection with a boundary.
                     try:
-                        intersection = seg.intersection(boundary.seg)
+                        intersection = line.intersect(boundary.line)
                     except ValueError:
                         continue
                     if intersection[TIME] == since:
@@ -546,12 +540,10 @@ class Gauge(object):
                     again = True  # iterate with same boundaries again.
                     bound, overlapped = boundary, True
                     since, value = intersection
-                    # adjust by more accurate value.
-                    if boundary.seg.velocity == 0:
-                        value = boundary.seg.value
-                    elif boundary.seg.since == since:
-                        value = boundary.seg.value
-                    value = clamp_by_boundaries(value, at=since)
+                    # clamp by boundaries.
+                    if bound is None or overlapped:
+                        value = min(value, ceil.line.guess(since))
+                        value = max(value, floor.line.guess(since))
                     yield (since, value)
                     break
                 if bound is not None:
@@ -559,22 +551,20 @@ class Gauge(object):
                 for boundary in walked_boundaries:
                     # find missing intersection caused by floating-point
                     # inaccuracy.
-                    bound_until = min(boundary.seg.until, until)
+                    bound_until = min(boundary.line.until, until)
                     if bound_until == +inf or bound_until < since:
                         continue
-                    boundary_value = boundary.seg.get(bound_until)
-                    if boundary.cmp_eq(seg.get(bound_until), boundary_value):
+                    boundary_value = boundary.line.get(bound_until)
+                    if boundary.cmp_eq(line.get(bound_until), boundary_value):
                         continue
                     bound, overlapped = boundary, True
                     since, value = bound_until, boundary_value
-                    value = clamp_by_boundaries(value, at=since)
                     yield (since, value)
                     break
             if until == +inf:
                 break
             # determine the last node in the current itreration.
             value += velocity * (until - since)
-            value = clamp_by_boundaries(value, at=until)
             yield (until, value)
             # prepare the next iteration.
             if method == ADD:
@@ -671,12 +661,27 @@ class Momentum(namedtuple('Momentum', ['velocity', 'since', 'until'])):
         return string
 
 
-class Segment(namedtuple('Segment', ['value', 'velocity', 'since', 'until'])):
+class Line(object):
+    """An abstract class to represent lines between 2 times which start from
+    `value`.  Subclasses should describe where lines end.
 
-    def __new__(cls, value, velocity, since=-inf, until=+inf):
-        value = float(value)
-        velocity = float(velocity)
-        return super(Segment, cls).__new__(cls, value, velocity, since, until)
+    .. note::
+
+       Each subclass must implement :meth:`_get`, :meth:`_earlier`,
+       :meth:`_later`, and :attr:`velocity` property.
+
+    """
+
+    since = None
+    until = None
+    value = None
+
+    velocity = NotImplemented
+
+    def __init__(self, since, until, value):
+        self.since = since
+        self.until = until
+        self.value = value
 
     def get(self, at=None):
         """Returns the value at the given time.
@@ -687,35 +692,51 @@ class Segment(namedtuple('Segment', ['value', 'velocity', 'since', 'until'])):
         if not self.since <= at <= self.until:
             raise ValueError('Out of the time range: {0:.2f}~{1:.2f}'
                              ''.format(self.since, self.until))
-        return self.value + self.velocity * (at - self.since)
+        return self._get(at)
 
     def guess(self, at=None):
-        """Same with :meth:`get` but it returns the first or last value if the
-        given time is out of the time range.
+        """Returns the value at the given time even the time it out of the time
+        range.
         """
         at = now_or(at)
         if at < self.since:
-            return self.value
-        elif self.until < at:
-            return self.get(self.until)
+            return self._earlier(at)
+        elif at > self.until:
+            return self._later(at)
         else:
             return self.get(at)
 
-    def intersection(self, seg):
-        """Gets the intersection with the given segment.
+    def _get(self, at):
+        """Implement at subclass as to calculate the value at the given time
+        which is between the time range.
+        """
+        raise NotImplementedError
+
+    def _earlier(self, at):
+        """Implement at subclass as to calculate the value at the given time
+        which is earlier than `since`.
+        """
+        raise NotImplementedError
+
+    def _later(self, at):
+        """Implement at subclass as to calculate the value at the given time
+        which is later than `until`.
+        """
+        raise NotImplementedError
+
+    def intersect(self, line):
+        """Gets the intersection with the given line.
 
         :raises ValueError: there's no intersection.
         """
-        # y-intercepts
-        y_intercept = (self.value - self.velocity * self.since)
-        seg_y_intercept = (seg.value - seg.velocity * seg.since)
+        intercept_delta = line.intercept() - self.intercept()
+        velocity_delta = self.velocity - line.velocity
         try:
-            time = (seg_y_intercept - y_intercept) / \
-                   (self.velocity - seg.velocity)
+            time = intercept_delta / velocity_delta
         except ZeroDivisionError:
-            raise ValueError('Parallel segment')
-        since = max(self.since, seg.since)
-        until = min(self.until, seg.until)
+            raise ValueError('Parallel line given')
+        since = max(self.since, line.since)
+        until = min(self.until, line.until)
         if since <= time <= until:
             pass
         else:
@@ -723,14 +744,79 @@ class Segment(namedtuple('Segment', ['value', 'velocity', 'since', 'until'])):
         value = self.get(time)
         return (time, value)
 
+    def intercept(self):
+        """Gets the value-intercept. (Y-intercept)"""
+        return self.value - self.velocity * self.since
+
+
+class Horizon(Line):
+    """A line which has no velocity."""
+
+    velocity = 0
+
+    def _get(self, at):
+        return self.value
+
+    def _earlier(self, at):
+        return self.value
+
+    def _later(self, at):
+        return self.value
+
+
+class Ray(Line):
+    """A line based on starting value and velocity."""
+
+    velocity = None
+
+    def __init__(self, since, until, value, velocity):
+        super(Ray, self).__init__(since, until, value)
+        self.velocity = velocity
+
+    def _get(self, at):
+        return self.value + self.velocity * (at - self.since)
+
+    def _earlier(self, at):
+        return self.value
+
+    def _later(self, at):
+        return self._get(self.until)
+
+
+class Segment(Line):
+    """A line based on starting and ending value."""
+
+    #: The value at `until`.
+    final = None
+
+    @property
+    def velocity(self):
+        value_delta = self.final - self.value
+        time_delta = self.until - self.since
+        return value_delta / time_delta
+
+    def __init__(self, since, until, value, final):
+        super(Segment, self).__init__(since, until, value)
+        self.final = final
+
+    def _get(self, at):
+        rate = float(at - self.since) / (self.until - self.since)
+        return self.value + rate * (self.final - self.value)
+
+    def _earlier(self, at):
+        return self.value
+
+    def _later(self, at):
+        return self.final
+
 
 class Boundary(object):
 
-    #: The current segment.  To select next segment, call :meth:`walk`.
-    seg = None
+    #: The current line.  To select next line, call :meth:`walk`.
+    line = None
 
-    #: The segment iterator.
-    segs_iter = None
+    #: The iterator of lines.
+    lines_iter = None
 
     #: Compares two values.  Choose one of `operator.lt` and `operator.gt`.
     cmp = None
@@ -740,16 +826,16 @@ class Boundary(object):
     #: `operator.gt` indicates :func:`max`.
     best = None
 
-    def __init__(self, segs_iter, cmp=operator.lt):
+    def __init__(self, lines_iter, cmp=operator.lt):
         assert cmp in [operator.lt, operator.gt]
-        self.segs_iter = segs_iter
+        self.lines_iter = lines_iter
         self.cmp = cmp
         self.best = {operator.lt: min, operator.gt: max}[cmp]
         self.walk()
 
     def walk(self):
-        """Choose the next segment."""
-        self.seg = next(self.segs_iter)
+        """Choose the next line."""
+        self.line = next(self.lines_iter)
 
     def cmp_eq(self, x, y):
         return x == y or self.cmp(x, y)
@@ -758,5 +844,5 @@ class Boundary(object):
         return x != y and not self.cmp(x, y)
 
     def __repr__(self):
-        return ('<{0} seg={1}, cmp={2}>'
-                ''.format(type(self).__name__, self.seg, self.cmp))
+        return ('<{0} line={1}, cmp={2}>'
+                ''.format(type(self).__name__, self.line, self.cmp))
